@@ -1,18 +1,22 @@
-package # hide from PAUSE 
+package # hide from PAUSE
     DBIx::Class::CDBICompat::AttributeAPI;
 
+use strict;
+use warnings;
+
+use base 'DBIx::Class';
+
 sub _attrs {
   my ($self, @atts) = @_;
   return @{$self->{_column_data}}{@atts};
 }
 
-*_attr = \&_attrs;
+sub _attr { shift->_attrs(@_) }
 
 sub _attribute_store {
   my $self   = shift;
   my $vals   = @_ == 1 ? shift: {@_};
-  my (@cols) = keys %$vals;
-  @{$self->{_column_data}}{@cols} = @{$vals}{@cols};
+  $self->store_column($_, $vals->{$_}) for keys %{$vals};
 }
 
 sub _attribute_set {
@@ -28,7 +32,7 @@ sub _attribute_delete {
 
 sub _attribute_exists {
   my ($self, $attr) = @_;
-  exists $self->{_column_data}{$attr};
+  $self->has_column_loaded($attr);
 }
 
 1;
