@@ -4,7 +4,10 @@ package
 use strict;
 use warnings;
 
+use base 'DBIx::Class';
+
 use DBIx::Class::_Util 'quote_sub';
+use namespace::clean;
 
 =head1 NAME
 
@@ -33,4 +36,17 @@ sub new {
     return bless $args, $class;
 }
 
+=head1 FURTHER QUESTIONS?
+
+Check the list of L<additional DBIC resources|DBIx::Class/GETTING HELP/SUPPORT>.
+
+=head1 COPYRIGHT AND LICENSE
+
+This module is free software L<copyright|DBIx::Class/COPYRIGHT AND LICENSE>
+by the L<DBIx::Class (DBIC) authors|DBIx::Class/AUTHORS>. You can
+redistribute it and/or modify it under the same terms as the
+L<DBIx::Class library|DBIx::Class/COPYRIGHT AND LICENSE>.
+
+=cut
+
 1;
