@@ -1,10 +1,10 @@
-package # hide from PAUSE 
+package # hide from PAUSE
     DBIx::Class::CDBICompat::Stringify;
 
 use strict;
 use warnings;
 
-use Scalar::Util;
+use base 'DBIx::Class';
 
 use overload
   '""' => sub { return shift->stringify_self; },
