@@ -3,9 +3,10 @@ package # hide from PAUSE
 
 use strict;
 use warnings;
-use base qw/Class::Data::Inheritable/;
+use base 'DBIx::Class';
 
 use Carp;
+use namespace::clean;
 
 __PACKAGE__->mk_classdata('_temp_columns' => { });
 
