@@ -3,7 +3,7 @@ package # hide from PAUSE
 
 use strict;
 use warnings;
-use Try::Tiny;
+use DBIx::Class::_Util 'dbic_internal_try';
 use namespace::clean;
 
 our %_pod_inherit_config =
@@ -15,21 +15,8 @@ sub has_many {
   my ($class, $rel, $f_class, $cond, $attrs) = @_;
 
   unless (ref $cond) {
-    $class->ensure_class_loaded($f_class);
-    my ($pri, $too_many) = try { $class->_pri_cols }
-      catch {
-        $class->throw_exception("Can't infer join condition for '$rel' on ${class}: $_");
-      };
 
-    $class->throw_exception(
-      "has_many can only infer join for a single primary key; ".
-      "${class} has more"
-    ) if $too_many;
-
-    $class->throw_exception(
-      "has_many needs a primary key to infer a join; ".
-      "${class} has none"
-    ) if !defined $pri && (!defined $cond || !length $cond);
+    my $pri = $class->result_source->_single_pri_col_or_die;
 
     my ($f_key,$guess);
     if (defined $cond && length $cond) {
@@ -38,13 +25,16 @@ sub has_many {
     } else {
       $class =~ /([^\:]+)$/;  # match is safe - $class can't be ''
       $f_key = lc $1; # go ahead and guess; best we can do
-      $guess = "using our class name '$class' as foreign key";
+      $guess = "using our class name '$class' as foreign key source";
     }
 
-    my $f_class_loaded = try { $f_class->columns };
-    $class->throw_exception(
-      "No such column '$f_key' on foreign class ${f_class} ($guess)"
-    ) if $f_class_loaded && !$f_class->has_column($f_key);
+# FIXME - this check needs to be moved to schema-composition time...
+#    # only perform checks if the far side appears already loaded
+#    if (my $f_rsrc = dbic_internal_try { $f_class->result_source } ) {
+#      $class->throw_exception(
+#        "No such column '$f_key' on foreign class ${f_class} ($guess)"
+#      ) if !$f_rsrc->has_column($f_key);
+#    }
 
     $cond = { "foreign.${f_key}" => "self.${pri}" };
   }
@@ -56,6 +46,7 @@ sub has_many {
     join_type => 'LEFT',
     cascade_delete => $default_cascade,
     cascade_copy => $default_cascade,
+    is_depends_on => 0,
     %{$attrs||{}}
   });
 }
