@@ -4,7 +4,7 @@ package # hide from PAUSE
 use strict;
 use warnings;
 use DBIx::Class::Carp;
-use Try::Tiny;
+use DBIx::Class::_Util qw( dbic_internal_try dbic_internal_catch );
 use namespace::clean;
 
 our %_pod_inherit_config =
@@ -23,7 +23,7 @@ sub has_one {
 sub _has_one {
   my ($class, $join_type, $rel, $f_class, $cond, $attrs) = @_;
   unless (ref $cond) {
-    my $pri = $class->result_source_instance->_single_pri_col_or_die;
+    my $pri = $class->result_source->_single_pri_col_or_die;
 
     my ($f_key,$guess,$f_rsrc);
     if (defined $cond && length $cond) {
@@ -34,13 +34,13 @@ sub _has_one {
       # at this point we need to load the foreigner, expensive or not
       $class->ensure_class_loaded($f_class);
 
-      $f_rsrc = try {
-        my $r = $f_class->result_source_instance;
+      $f_rsrc = dbic_internal_try {
+        my $r = $f_class->result_source;
         die "There got to be some columns by now... (exception caught and rewritten by catch below)"
           unless $r->columns;
         $r;
       }
-      catch {
+      dbic_internal_catch {
         $class->throw_exception(
           "Foreign class '$f_class' does not seem to be a Result class "
         . "(or it simply did not load entirely due to a circular relation chain)"
@@ -59,8 +59,8 @@ sub _has_one {
 
 # FIXME - this check needs to be moved to schema-composition time...
 #    # only perform checks if the far side was not preloaded above *AND*
-#    # appears to have been loaded by something else (has a rsrc_instance)
-#    if (! $f_rsrc and $f_rsrc = try { $f_class->result_source_instance }) {
+#    # appears to have been loaded by something else (has a rsrc)
+#    if (! $f_rsrc and $f_rsrc = dbic_internal_try { $f_class->result_source }) {
 #      $class->throw_exception(
 #        "No such column '$f_key' on foreign class ${f_class} ($guess)"
 #      ) if !$f_rsrc->has_column($f_key);
@@ -77,6 +77,7 @@ sub _has_one {
    { accessor => 'single',
      cascade_update => $default_cascade,
      cascade_delete => $default_cascade,
+     is_depends_on => 0,
      ($join_type ? ('join_type' => $join_type) : ()),
      %{$attrs || {}} });
   1;
@@ -95,12 +96,18 @@ sub _validate_has_one_condition {
     return unless $self_id =~ /^self\.(.*)$/;
 
     my $key = $1;
-    $class->throw_exception("Defining rel on ${class} that includes '$key' but no such column defined here yet")
-        unless $class->has_column($key);
-    my $column_info = $class->column_info($key);
-    if ( $column_info->{is_nullable} ) {
-      carp(qq'"might_have/has_one" must not be on columns with is_nullable set to true ($class/$key). This might indicate an incorrect use of those relationship helpers instead of belongs_to.');
-    }
+
+    my $column_info = $class->result_source->columns_info->{$key}
+      or $class->throw_exception(
+        "Defining rel on ${class} that includes '$key' "
+      . 'but no such column defined there yet'
+      );
+
+    carp(
+      "'might_have'/'has_one' must not be used on columns with is_nullable "
+    . "set to true ($class/$key). This almost certainly indicates an "
+    . "incorrect use of these relationship helpers instead of 'belongs_to'"
+    ) if $column_info->{is_nullable};
   }
 }
 
