@@ -6,8 +6,7 @@ use strict;
 use base 'DBIx::Class::Storage::DBI';
 use mro 'c3';
 
-use Sub::Name;
-use Try::Tiny;
+use DBIx::Class::_Util qw( sigwarn_silencer modver_gt_or_eq set_subname );
 use namespace::clean;
 
 =head1 NAME
@@ -29,12 +28,9 @@ sub _rebless { shift->_determine_connector_driver('ADO') }
 sub _dbh_get_info {
   my $self = shift;
 
-  my $warn_handler = $SIG{__WARN__} || sub { warn @_ };
-
-  local $SIG{__WARN__} = sub {
-    $warn_handler->(@_)
-      unless $_[0] =~ m{^Missing argument in sprintf at \S+/ADO/GetInfo\.pm};
-  };
+  local $SIG{__WARN__} = sigwarn_silencer(
+    qr{^Missing argument in sprintf at \S+/ADO/GetInfo\.pm}
+  );
 
   $self->next::method(@_);
 }
@@ -47,16 +43,14 @@ sub _init {
   unless ($DBD::ADO::__DBIC_MONKEYPATCH_CHECKED__) {
     require DBD::ADO;
 
-    unless (try { DBD::ADO->VERSION('2.99'); 1 }) {
+    unless ( modver_gt_or_eq( 'DBD::ADO', '2.99' ) ) {
       no warnings 'redefine';
       my $disconnect = *DBD::ADO::db::disconnect{CODE};
 
-      *DBD::ADO::db::disconnect = subname 'DBD::ADO::db::disconnect' => sub {
-        my $warn_handler = $SIG{__WARN__} || sub { warn @_ };
-        local $SIG{__WARN__} = sub {
-          $warn_handler->(@_)
-            unless $_[0] =~ /Not a Win32::OLE object|uninitialized value/;
-        };
+      *DBD::ADO::db::disconnect = set_subname 'DBD::ADO::db::disconnect' => sub {
+        local $SIG{__WARN__} = sigwarn_silencer(
+          qr/Not a Win32::OLE object|uninitialized value/
+        );
         $disconnect->(@_);
       };
     }
@@ -79,15 +73,19 @@ sub _init {
 #  $sth;
 #}
 
-1;
+=head1 FURTHER QUESTIONS?
 
-=head1 AUTHOR
+Check the list of L<additional DBIC resources|DBIx::Class/GETTING HELP/SUPPORT>.
 
-See L<DBIx::Class/AUTHOR> and L<DBIx::Class/CONTRIBUTORS>.
+=head1 COPYRIGHT AND LICENSE
 
-=head1 LICENSE
-
-You may distribute this code under the same terms as Perl itself.
+This module is free software L<copyright|DBIx::Class/COPYRIGHT AND LICENSE>
+by the L<DBIx::Class (DBIC) authors|DBIx::Class/AUTHORS>. You can
+redistribute it and/or modify it under the same terms as the
+L<DBIx::Class library|DBIx::Class/COPYRIGHT AND LICENSE>.
 
 =cut
+
+1;
+
 # vim:sts=2 sw=2:
