@@ -7,9 +7,6 @@ use base 'DBIx::Class::Storage::DBI';
 use mro 'c3';
 
 use DBIx::Class::SQLMaker::LimitDialects;
-use List::Util qw/first/;
-
-use namespace::clean;
 
 =head1 NAME
 
@@ -99,7 +96,7 @@ sub interpolate_unquoted {
       and
     $_[1]
       and
-    $_[2] !~ /\D/
+    $_[2] !~ /[^0-9]/
       and
     $_[1] =~ /int(?:eger)? | (?:tiny|small|medium|big)int/ix
   );
@@ -119,13 +116,16 @@ sub _prep_interpolated_value {
   return $_[2];
 }
 
-=head1 AUTHORS
+=head1 FURTHER QUESTIONS?
 
-See L<DBIx::Class/CONTRIBUTORS>
+Check the list of L<additional DBIC resources|DBIx::Class/GETTING HELP/SUPPORT>.
 
-=head1 LICENSE
+=head1 COPYRIGHT AND LICENSE
 
-You may distribute this code under the same terms as Perl itself.
+This module is free software L<copyright|DBIx::Class/COPYRIGHT AND LICENSE>
+by the L<DBIx::Class (DBIC) authors|DBIx::Class/AUTHORS>. You can
+redistribute it and/or modify it under the same terms as the
+L<DBIx::Class library|DBIx::Class/COPYRIGHT AND LICENSE>.
 
 =cut
 
