@@ -2,7 +2,7 @@ package DBIx::Class::Storage::DBI::Sybase;
 
 use strict;
 use warnings;
-use Try::Tiny;
+use DBIx::Class::_Util qw( dbic_internal_try dbic_internal_catch );
 use namespace::clean;
 
 use base qw/DBIx::Class::Storage::DBI/;
@@ -21,28 +21,26 @@ L<DBD::Sybase>
 
 =cut
 
-sub _rebless {
+sub _rebless { shift->_determine_connector_driver('Sybase') }
+
+sub _get_rdbms_name {
   my $self = shift;
 
-  my $dbtype;
-  try {
-    $dbtype = @{$self->_get_dbh->selectrow_arrayref(qq{sp_server_info \@attribute_id=1})}[2]
-  } catch {
+  dbic_internal_try {
+    my $name = $self->_get_dbh->selectrow_arrayref('sp_server_info @attribute_id=1')->[2];
+
+    if ($name) {
+      $name =~ s/\W/_/gi;
+
+      # saner class name
+      $name = 'ASE' if $name eq 'SQL_Server';
+    }
+
+    $name;  # RV
+  }
+  dbic_internal_catch {
     $self->throw_exception("Unable to establish connection to determine database type: $_")
   };
-
-  if ($dbtype) {
-    $dbtype =~ s/\W/_/gi;
-
-    # saner class name
-    $dbtype = 'ASE' if $dbtype eq 'SQL_Server';
-
-    my $subclass = __PACKAGE__ . "::$dbtype";
-    if ($self->load_optional_class($subclass)) {
-      bless $self, $subclass;
-      $self->_rebless;
-    }
-  }
 }
 
 sub _init {
@@ -78,26 +76,10 @@ sub _ping {
   local $dbh->{RaiseError} = 1;
   local $dbh->{PrintError} = 0;
 
-# FIXME if the main connection goes stale, does opening another for this statement
-# really determine anything?
-
-  if ($dbh->{syb_no_child_con}) {
-    return try {
-      $self->_connect->do('select 1');
-      1;
-    }
-    catch {
-      0;
-    };
-  }
-
-  return try {
-    $dbh->do('select 1');
-    1;
-  }
-  catch {
-    0;
-  };
+  ( dbic_internal_try { $dbh->do('select 1'); 1 } )
+    ? 1
+    : 0
+  ;
 }
 
 sub _set_max_connect {
