@@ -11,8 +11,8 @@ __PACKAGE__->table('event');
 
 __PACKAGE__->add_columns(
   id => { data_type => 'integer', is_auto_increment => 1 },
-  starts_at => { data_type => 'datetime', timezone => "America/Chicago", locale => 'de_DE' },
-  created_on => { data_type => 'timestamp with time zone', timezone => "America/Chicago" },
+  starts_at => { data_type => 'datetime', time_zone => "America/Chicago", locale => 'de_DE' },
+  created_on => { data_type => 'timestamp with time zone', time_zone => "America/Chicago" },
   ts_without_tz => { data_type => 'timestamp without time zone' },
 );
 
