@@ -7,6 +7,8 @@ use strict;
 use base 'DBICTest::BaseResult';
 use DBICTest::Util 'check_customcond_args';
 
+# The component order is Part of a test,
+# important to remain as-is
 __PACKAGE__->load_components(qw{
     +DBICTest::DeployComponent
     InflateColumn::DateTime
@@ -67,7 +69,7 @@ sub {
     },
 
     ! $args->{self_result_object} ? () : {
-     "$args->{foreign_alias}.cdid" => $args->{self_result_object}->cd
+     "$args->{foreign_alias}.cdid" => $args->{self_result_object}->get_column('cd')
     },
 
     ! $args->{foreign_values} ? () : {
@@ -116,6 +118,20 @@ __PACKAGE__->has_many (
   }
 );
 
+__PACKAGE__->has_many (
+  deliberately_broken_all_cd_tracks => __PACKAGE__,
+  sub {
+    # This is for test purposes only. A regular user does not
+    # need to sanity check the passed-in arguments, this is what
+    # the tests are for :)
+    my $args = &check_customcond_args;
+
+    return {
+      "$args->{foreign_alias}.cd" => "$args->{self_alias}.cd"
+    };
+  }
+);
+
 our $hook_cb;
 
 sub sqlt_deploy_hook {
